<?php
namespace Drupal\asc_courses\Plugin\migrate\source;

use Drupal\Core\Database\Query\Condition;
use Drupal\migrate\Row;
// use Drupal\migrate\MigrateSkipRowException;
// use Drupal\migrate_drupal\Plugin\migrate\source\d7\FieldableEntity;
use Drupal\migrate\Plugin\migrate\source\SqlBase;

/**
 * Drupal 7 file_entity source from database.
 *
 * @MigrateSource(
 *   id = "asc_person",
 *   source_provider = "user"
 * )
 */
class CourseDataCached extends SqlBase {

  /**
   * {@inheritdoc}
   */
  public function query() {
    //echo "AscPerson::query()\n";

    /*
      -- Users who also have people nodes (where clearly identifiable)
      select
        u.uid,
        u.mail,
        substring(u.mail, 1, locate('@osu.edu', u.mail)-1) as umail,
        u.name,
        substring(u.name, 1, locate('@osu.edu', u.name)-1) as uname,
        fdfape.entity_id,
        fdfapp.entity_id,
        fdfape.field_asc_people_email_email,
        fdfapp.field_asc_people_picture_opic
      from
        users u
          left outer join field_data_field_asc_people_email fdfape
            on u.mail = fdfape.field_asc_people_email_email
          left outer join field_data_field_asc_people_picture fdfapp
            on fdfape.entity_id = fdfapp.entity_id
      ;


      select
        substring(u.mail, 1, locate('@osu.edu', u.mail)-1) as umail
      from users u
      union
      select field_asc_people_picture_opic
      from field_data_field_asc_people_picture fdfapp
      ;

      select
        u.mail as umail
      from users u
      union
      select field_asc_people_picture_opic
      from field_data_field_asc_people_picture;

      select uid, name, mail, created, access, login, status from users limit 20;

        select

        from
          users u
    */

    $course_data_query = $this->select('asc_course_data', 'acd');
    $course_data_query->fields('acd', ['id', 'date', 'dept_org', 'raw_json']);
    // $course_data_query->



    $people_query = $this->select('node', 'n');
    $people_query->addExpression('n.nid', 'asc_people_id');
    $people_query->fields('n', array('title', 'created', 'changed', 'status'));
    $people_query->leftJoin('field_data_field_asc_people_picture', 'fdfapp', 'n.nid = fdfapp.entity_id');
    $people_query->leftJoin('field_data_field_asc_people_email', 'fdfape', 'n.nid = fdfape.entity_id');
    $people_query->addExpression('fdfapp.field_asc_people_picture_opic', 'osu_name_num');
    $people_query->addExpression('fdfape.field_asc_people_email_email', 'name');
    $people_query->addExpression('fdfape.field_asc_people_email_email', 'mail');
    $people_query->addExpression('0', 'login');
    $people_query->addExpression('0', 'access');
    $people_query->condition('n.status', 0, '>');
    $people_query->condition('n.type', 'asc_people', '=');
    // echo "people query: $people_query\n";

    // $pictures_query = $this->select('field_data_field_asc_people_picture', 'fdfapp');
    // $pictures_query->addExpression("fdfapp.entity_id", 'asc_people_id');
    // $pictures_query->addExpression("'node'", "entity_type");
    // $pictures_query->addExpression("fdfapp.entity_id", "entity_id");
    // $pictures_query->addExpression("concat(fdfapp.field_asc_people_picture_opic, '@osu.edu')", "name");
    // $pictures_query->addExpression("concat(fdfapp.field_asc_people_picture_opic, '@osu.edu')", "mail");
    // $pictures_query->addExpression("fdfapp.field_asc_people_picture_opic", "osu_name_num");

    // $pictures_query->addExpression("1", "status");
    // $pictures_query->addExpression("unix_timestamp()", "created");
    // $pictures_query->range(1,1);
    // echo "pictures query: $pictures_query\n";

    return $people_query;
  }

  /**
   * {@inheritdoc}
   */
  public function count($refresh = false) {
    //echo "AscPerson::count()\n";

    $peoplecount_query = $this->select('node');
    $peoplecount_query->condition('type', 'asc_people', '=');
    $peoplecount_query->condition('status', 0, '>');
    $peoplecount_query->addExpression('count(*)', 'count');
    $peoplecount = $peoplecount_query->execute()->fetchField(0);

    return $peoplecount;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row) {
    // print_r($row);
    $mail = $row->getSourceProperty('mail');
    if(empty($mail)) {
      $osu_name_num = $row->getSourceProperty('osu_name_num');
      $mail = $osu_name_num . "@osu.edu";
      $row->setSourceProperty('mail', $mail);
      echo "Update email using OSU name.#: $mail\n";
    }

    foreach (array_keys($this->getFields('node', 'asc_people')) as $field) {
      $nid = $row->getSourceProperty('asc_people_id');
      $row->setSourceProperty($field, $this->getFieldValues('node', $field, $nid));
    }

    $row->setSourceProperty('roles', ['department_faculty_staff']);

    // $node_query = $this->select('node', 'n')
    //   ->fields('n', array('created', 'changed'))
    //   ->condition('n.nid', $nid);
    // $node_query->addExpression('0', 'login');
    // $node_query->addExpression('0', 'access');
    // $node_row = $node_query->execute()->fetch();

    // foreach($node_row as $field_name => $field_value) {
    //   $row->setSourceProperty($field_name, $field_value);
    // }

    return parent::prepareRow($row);
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    $fields = [
      'uid' => $this->t('User ID'),
      'name' => $this->t('Username'),
      //'pass' => $this->t('Password'),
      'mail' => $this->t('Email address'),
      //'signature' => $this->t('Signature'),
      //'signature_format' => $this->t('Signature format'),
      'created' => $this->t('Registered timestamp'),
      'access' => $this->t('Last access timestamp'),
      'login' => $this->t('Last login timestamp'),
      'changed' => $this->t('Last time user was updated'),
      'status' => $this->t('Status'),
      'timezone' => $this->t('Timezone'),
      'language' => $this->t('Language'),
      'picture' => $this->t('Picture'),
      'init' => $this->t('Init'),
      'data' => $this->t('User data'),
      'roles' => $this->t('Roles'),
    ];

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {
    /*
    $ids['fid']['type'] = 'integer';
    return $ids;
    return [
      'uid' => [
        'type' => 'integer',
        'alias' => 'u',
      ],
    ];
    */
    $ids = [
      'asc_people_id' => [
        'type' => 'string'
      ]
    ];

    return $ids;
  }
}

